#include "templates.hpp"
#include "config.hpp"   
#include "utils/directories.hpp"
#include <filesystem>
#include <iostream>
#include <vector>
#include <string>
#include <algorithm>
#include <iomanip>

namespace dropshell {


bool get_templates(std::vector<template_info>& templates) {
    templates.clear();
        
    // Helper function to add templates from a directory
    auto add_templates_from_dir = [&templates](const std::string& dir_path) {
        if (!std::filesystem::exists(dir_path)) {
            return;
        }
        
        for (const auto& entry : std::filesystem::directory_iterator(dir_path)) {
            if (entry.is_directory()) {
                template_info info;
                info.name = entry.path().filename().string();
                info.path = entry.path().string();
                
                // Check if template with same name already exists
                bool found = false;
                auto it = std::find_if(templates.begin(), templates.end(),
                    [&info](const template_info& t) { return t.name == info.name; });
                found = (it!=templates.end());
                found |= info.name=="example"; // don't include the example template!

                if (!found) {
                    templates.push_back(info);
                }
            }
        }
    };
    
    add_templates_from_dir(get_local_config_templates_path());
    add_templates_from_dir(get_local_system_templates_path());    
    
    return true;
}

bool get_template_info(const std::string& name, template_info& info) {
    std::vector<template_info> templates;
    if (!get_templates(templates)) {
        return false;
    }
    
    auto it = std::find_if(templates.begin(), templates.end(),
        [&name](const template_info& t) { return t.name == name; });
    
    if (it != templates.end()) {
        info = *it;
        return true;
    }
    
    return false;
}

bool template_command_exists(const std::string &template_name, const std::string &command)
{
    template_info info;
    if (!get_template_info(template_name, info)) {
        return false;
    }
    
    std::string path = info.path + "/" + command + ".sh";
    return (std::filesystem::exists(path));
}

void list_templates() {
    std::vector<template_info> templates;
    
    if (!get_templates(templates)) {
        std::cerr << "Error: Failed to get templates" << std::endl;
        return;
    }
    
    if (templates.empty()) {
        std::cout << "No templates found." << std::endl;
        return;
    }
    
    std::cout << "Available templates:" << std::endl;
    std::cout << std::left << std::setw(20) << "Name" << "Path" << std::endl;
    std::cout << std::string(60, '-') << std::endl;
    
    for (const auto& t : templates) {
        std::cout << std::left << std::setw(20) << t.name << t.path << std::endl;
    }
} 

} // namespace dropshell